/**
 * DraggableMail.js - Extensions to the stock Scriptaculous Draggable library
 * which allows us to do click-selections, move multiple items, DIMP-specific
 * update graphics, etc.
 *
 * $Horde: dimp/js/src/DraggableMail.js,v 1.32 2007/06/29 21:21:44 jan Exp $
 *
 * Copyright 2005-2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

// For DIMP, we need to take into account the scroll bar since the folder list
// may exceed the available size and cause the drop zones to move offscreen.
// Requires the parameter 'scroll' to be true when passed into add(). Also,
// will ignore Draggable elements that have a non-empty 'nodrop' property.
Droppables.isAffected = function(point, element, drop) {
    return (
        !element.nodrop &&
        (drop.element != element) &&
         ((!drop._containers) ||
          this.isContained(element, drop)) &&
         ((!drop.accept) ||
          (Element.classNames(element).detect(
           function(v) { return drop.accept.include(v) } ) )) &&
         ((drop.scroll) ?
             Position.withinIncludingScrolloffsets(drop.element, point[0], point[1]) :
             Position.within(drop.element, point[0], point[1])) );
};

var DraggableMail = Class.create();
Object.extend(Object.extend(DraggableMail.prototype, Draggable.prototype), {
    // Overrides the builtin Draggable effects.
    draw: function(e) {
        var m = $('mousedrag');
        if (this.element.id == m.id) {
            return;
        }

        if (++this.movethreshold < 5) {
            return;
        }

        m.setStyle({ left: (e[0] + 15) + 'px',
                     top: (e[1] + 10) + 'px',
                     zIndex: 10001 });
        $('dragcount').update(DimpCore.selectedCount());
        m.show();
    },

    // Need to redefine this because the original will stop the Event (which
    // we don't want until after onStart), won't call onStart (which we do
    // want), ignores right clicks, and won't init the move threshold.
    initDrag: function(e) {
        if (typeof Draggable._dragging[this.element] != 'undefined' &&
            Draggable._dragging[this.element]) {
            return;
        }

        // abort on form elements, fixes a Firefox issue
        var tag_name = Event.element(e).tagName.toUpperCase();
        if (tag_name &&
            (tag_name == 'INPUT' ||
             tag_name == 'SELECT' ||
             tag_name == 'OPTION' ||
             tag_name == 'BUTTON' ||
             tag_name == 'TEXTAREA')) {
             return;
        }

        var pointer = [Event.pointerX(e), Event.pointerY(e)],
            pos = Position.cumulativeOffset(this.element);
        this.offset = [0,1].map( function(i) { return (pointer[i] - pos[i]) });

        Draggables.activate(this);
        Draggables.notify('onStart', this, e);
        Event.stop(e);
        this.movethreshold = 0;
    },

    // Need to redefine or else onEnd won't be triggered when we click &
    // immediately release.
    endDrag: function(e) {
        this.stopScrolling();
        this.finishDrag(e, true);
        Event.stop(e);
    }
});

var DraggableMailOpts = {
    endeffect: false,
    scroll: 'normalfolders',
    onStart: function(d, e) {
        // We only care about 'mousedown' events in this code.
        if (e.type != 'mousedown') {
            return;
        }

        var id = d.element.id.substring(3);

        // Handle selection first.
        if (e.ctrlKey || e.metaKey) {
            DimpCore.ctrlSelect(id, e);
        } else if (e.shiftKey) {
            DimpCore.shiftSelect(id, e);
        } else {
            if (DimpCore.isSelected(id)) {
                d.selectIfNoDrag = true;
                d.wasDragged = false;
            } else {
                DimpCore.messageClick(id, e);
            }
        }

        if (!Event.isLeftClick(e)) {
            d.selectIfNoDrag = d.wasDragged = false;
            return;
        }
    },
    onDrag: function(d, e) {
        d.wasDragged = true;
    },
    onEnd: function(d, e) {
        var id = d.element.id.substring(3);

        if (d.selectIfNoDrag && !d.wasDragged) {
            DimpCore.messageClick(id, e);
        }

        d.selectIfNoDrag = d.wasDragged = false;
        $('mousedrag').hide();
    }
};
