<?php

$block_name = _("Calendar");

/**
 * Horde_Block_Dimp_calendar:: customized Kronolith block for DIMP.
 *
 * $Horde: dimp/lib/Block/calendar.php,v 1.13 2007/06/28 18:57:33 slusarz Exp $
 *
 * Copyright 2005-2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @package Horde_Block
 */
class Horde_Block_dimp_calendar extends Horde_Block {

    var $_app = 'kronolith';

    /**
     * The content to go in this block.
     *
     * @return string   The content
     */
    function _content()
    {
        global $registry, $prefs;

        @define('KRONOLITH_BASE', $registry->get('fileroot', 'kronolith'));
        require_once KRONOLITH_BASE . '/lib/base.php';
        require_once KRONOLITH_BASE . '/lib/Day.php';
        require_once 'Horde/Prefs/CategoryManager.php';

        Horde::addScriptFile('tooltip.js', 'horde', true);

        $now = time();
        $today = date('j');

        $startDate = new Horde_Date(array('year' => date('Y'),
                                          'month' => date('n'),
                                          'mday' => date('j')));
        $endDate = new Horde_Date(array('year' => date('Y'),
                                        'month' => date('n'),
                                        'mday' => date('j') + $prefs->getValue('summary_days')));
        $endDate->correct();

        if (isset($this->_params['calendar']) &&
            $this->_params['calendar'] != '__all') {
            $allevents = Kronolith::listEvents($startDate,
                                               $endDate,
                                               array($this->_params['calendar']));
        } else {
            $allevents = Kronolith::listEvents($startDate,
                                               $endDate,
                                               $GLOBALS['display_calendars']);
        }
        if (is_a($allevents, 'PEAR_Error')) {
            return '<em>' . $allevents->getMessage() . '</em>';
        }

        $html = '';
        $iMax = $today + $prefs->getValue('summary_days');
        $firstday = true;
        for ($i = $today; $i < $today + $iMax; ++$i) {
            $day = new Kronolith_Day(date('n'), $i);
            if (empty($allevents[$day->getStamp()])) {
                continue;
            }

            $events = &$allevents[$day->getStamp()];
            $firstevent = true;

            $today12am = mktime(0, 0, 0,
                                $day->month,
                                $day->mday,
                                $day->year);
            $tomorrow12am = mktime(0, 0, 0,
                                   $day->month,
                                   $day->mday + 1,
                                   $day->year);
            foreach ($events as $event) {
                if ($event->recurs()) {
                    $event->start = new Horde_Date(
                        array('hour' => $event->start->hour,
                              'min' => $event->start->min,
                              'sec' => $event->start->sec,
                              'month' => $day->month,
                              'mday' => $day->mday,
                              'year' => $day->year));
                    $event->end = new Horde_Date($event->start->timestamp() +
                                                 ($event->durMin * 60));
                } else {
                    if ($event->start->timestamp() < $today12am) {
                        $event->start = new Horde_Date($today12am);
                    }
                    if ($event->end->timestamp() >= $tomorrow12am) {
                        $event->end = new Horde_Date($tomorrow12am);
                    }
                }
                if ($event->end->timestamp() < $now ||
                    ($prefs->getValue('summary_alarms') && !$event->alarm)) {
                    continue;
                }
                if ($firstevent) {
                    if (!$firstday) {
                        $html .= '<tr><td class="monthgrid-spacer" colspan="4">&nbsp;</td></tr>';
                    }
                    $html .= '<tr><td class="monthgrid-header" colspan="4"><strong>';
                    if ($day->isToday()) {
                        $dayname = _("Today");
                    } elseif ($day->isTomorrow()) {
                        $dayname = _("Tomorrow");
                    } elseif ($day->diff() < 7) {
                        $dayname = strftime('%A', $day->getStamp());
                    } else {
                        $dayname = strftime($prefs->getValue('date_format'),
                                            $day->getStamp());
                    }
                    $daylink = Horde::applicationUrl('day.php');
                    $daylink = Util::addParameter($daylink,
                                                  'timestamp',
                                                  $day->getStamp());
                    $html .= $dayname . '</strong></td></tr>';
                    $firstevent = false;
                    $firstday = false;
                }

                if ($event->start->hour != 0 ||
                    $event->start->min != 0 ||
                    (($event->end->timestamp() - $event->start->timestamp()) %
                     (24 * 60 * 60)) != 0) {
                    if ($prefs->getValue('twentyFour')) {
                        $start = date('G:i', $event->start->timestamp());
                        $end = date('G:i', $event->end->timestamp());
                    } else {
                        $start = date('g:i A', $event->start->timestamp());
                        $end = date('g:i A', $event->end->timestamp());
                    }
                } else {
                    $start = _("All day");
                    $end = '&nbsp;';
                }

                $html .= '<tr class="monthgrid-row"><td>&rarr;</td>' .
                    '<td class="monthgrid-row-right">' . $start . '</td><td class="monthgrid-row-right">' . $end . '</td>' .
                    '<td>' . preg_replace('/<a href="([^"]+)"/', '<a onclick="DimpCore.go(\'kronolith\', \'$1\')" href="#" ', $event->getLink()) . '</td></tr>';
            }
        }

        if (empty($html)) {
            return '<em>' . _("No events to display") . '</em>';
        }

        return '<table class="block-monthgrid">' . $html . '</table>';
    }

}
